using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

/// @author Antti-Jussi
/// @version 2025
/// 
/// <summary>
/// Lukujen esiintymiskerrat.
/// </summary>
public class G12_LaskeEsiintymat
{
    /// <summary>
    /// 
    /// </summary>
    /// <param name="args">ei käytössä</param>
    public static void Main(string[] args)
    {
        Random r = new Random();
        int montako = 100;
        int kierroksia = 1000000;
        int[] luvut = Tayta(r, montako, 0, 11);

        Stopwatch sw = Stopwatch.StartNew();
        for (int i = 0; i < kierroksia; i++)
            LaskeEsiintymat(luvut);
        sw.Stop();
        double ms = sw.Elapsed.TotalMilliseconds;
        Console.WriteLine("long-menetelmällä hommaan meni " + ms + " millisekuntia.");

        sw = Stopwatch.StartNew();
        for (int i = 0; i < kierroksia; i++)
            LaskeEsiintymat2(luvut);
        sw.Stop();
        ms = sw.Elapsed.TotalMilliseconds;
        Console.WriteLine("Linq:lla hommaan meni " + ms + " millisekuntia.");

        sw = Stopwatch.StartNew();
        for (int i = 0; i < kierroksia; i++)
            LaskeEsiintymat3(luvut);
        sw.Stop();
        ms = sw.Elapsed.TotalMilliseconds;
        Console.WriteLine("Dictionarylla (ilman poistoa) hommaan meni " + ms + " millisekuntia.");

        ms = sw.Elapsed.TotalMilliseconds;
        Console.WriteLine("KAksiulotteisilla taulukoilla " + ms + " millisekuntia.");
    }

    /// <summary>
    /// Täytä satunnaisia lukuja taulukkoon
    /// </summary>
    /// <param name="r"></param>
    /// <param name="montako"></param>
    /// <param name="min"></param>
    /// <param name="max"></param>
    /// <returns></returns>
    private static int[] Tayta(Random r, int montako, int min, int max)
    {
        int[] t = new int[montako];
        for (int i = 0; i < t.Length; i++)
        {
            t[i] = r.Next(min, max);
        }
        return t;
    }

    /// <summary>
    /// Palautetaan luvut esiintymien mukaisessa järjestyksessä.
    /// Toteutus: Alkuperäisestä taulukosta tehdään
    /// ensin kopio long luvuiksi ja sitten se lajitellaan.  Kopioissa lasketaan
    /// samojen peräkkäisten alkioiden lukumäärä ja sitten pari
    /// laitetaan kopiotaulukon alkuun longina, jossa on (lkm,luku);
    /// Lievä pullonkaula: kokonaislukutaulukon järjestäminen.
    /// n on alkioiden määrä, m erilaisten alkioiden määrä.
    /// Kompleksisuus: n + n log n + n + m log m + m
    /// Olioita 2 kpl: long taulukko, palautustaulukko
    /// <summary>
    /// <param name="luvut">tutkittavat luvut</param>
    /// <returns>luvut esiintymien määrän mukaisessa järjestyksessä</returns>
    /// <example>
    /// <pre name="test">
    ///   int[] luvut = {1,2,3,34,34,2,1,34,1,1,1};
    ///   int[] tulos = LaskeEsiintymat(luvut);
    ///   String.Join(" ",tulos) === "3 2 34 1";
    ///   String.Join(" ", LaskeEsiintymat(new int[]{})) === "";
    ///   String.Join(" ", LaskeEsiintymat(new int[]{1})) === "1";
    ///   String.Join(" ", LaskeEsiintymat(new int[]{1,2,2})) === "1 2";
    /// </pre>
    /// </example>
    public static int[] LaskeEsiintymat(int[] luvut)
    {
        int n = luvut.Length;
        if (n == 0) return luvut;
        long[] parit = new long[n];
        int m = 0;
        // for (int i = 0; i < n; i++) parit[i] = luvut[i];
        { int i = 0; foreach (int luku in luvut) parit[i++] = luku; }
        Array.Sort(parit);
        long edellinen = parit[0];
        long lkm = 1;
        for (int i = 1; i < n; i++)
        {
            long luku = parit[i];
            if (luku > edellinen)
            {
                parit[m++] = edellinen + (lkm << 32);
                edellinen = luku;
                lkm = 1;
            }
            else lkm++;
        }
        parit[m++] = edellinen + (lkm << 32);
        Array.Sort(parit, 0, m);
        int[] tulos = new int[m];
        for (int i = 0; i < m; i++) tulos[i] = (int)parit[i];
        return tulos;
    }

    /// <summary>
    /// Funktio järjestää sille annetun taulukon arvojen esiintymiskertojen lukumäärän mukaan sekä muodostaa uuden taulukon 
    /// siten, että kukin arvo esiintyy vain kerran.
    /// </summary>
    /// <param name="taulukko">Käsiteltävä taulukko</param>
    /// <returns>Esiintymiskertojen mukaan järjestetty taulukko</returns>
    /// <example>
    /// <pre name="test">
    ///   int[] luvut = {1,2,3,34,34,2,1,34,1,1,1};
    ///   int[] tulos = LaskeEsiintymat2(luvut);
    ///   String.Join(" ",tulos) === "3 2 34 1";
    ///   String.Join(" ", LaskeEsiintymat2(new int[]{})) === "";
    ///   String.Join(" ", LaskeEsiintymat2(new int[]{1})) === "1";
    ///   String.Join(" ", LaskeEsiintymat2(new int[]{1,2,2})) === "1 2";
    /// </pre>
    /// </example>
    public static int[] LaskeEsiintymat2(int[] taulukko)
    {
        return taulukko.ToList<int>().GroupBy(i => i).OrderBy(g => g.Count()).Select(g => g.Key).ToArray<int>();
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="taulukko"></param>
    /// <returns></returns>
    /// <example>
    /// <pre name="test">
    ///   int[] luvut = {1,2,3,34,34,2,1,34,1,1,1};
    ///   int[] tulos = LaskeEsiintymat3(luvut);
    ///   String.Join(" ",tulos) === "3 2 34 1";
    ///   luvut = new int[] {1,2,2,3,34,34,2,1,34,1,1,1};
    ///   tulos = LaskeEsiintymat3(luvut);
    ///   String.Join(" ",tulos) === "3 2 34 1";
    ///   String.Join(" ", LaskeEsiintymat3(new int[]{})) === "";
    ///   String.Join(" ", LaskeEsiintymat3(new int[]{1})) === "1";
    ///   String.Join(" ", LaskeEsiintymat3(new int[]{1,2,2})) === "1 2";
    /// </pre>
    /// </example>
    public static int[] LaskeEsiintymat3(int[] taulukko)
    {
        // Muutetaan luvut-taulukko ensin listaksi, jotta olisi helpompi kasitella "jaljella olevia lukuja"
        List<int> lista = taulukko.ToList<int>();
        lista.Sort();

        if (lista.Count == 0) return new int[] { };

        // Dictionaryyn tallennetaan avain-arvopareja.
        // Tässä siis avain = parametrina annetun taulukon alkio
        //            arvo  = kuinka monta kpl kyseistä alkiota on löytynyt
        Dictionary<int, int> alkiotJaMaarat = new Dictionary<int, int>();
        int lukuJotaEtsitaan = lista[0];
        int montakoLoydetty = 1;
        int i = 1;
        while (i < lista.Count)
        {
            if (lista[i] != lukuJotaEtsitaan)
            {
                alkiotJaMaarat.Add(lukuJotaEtsitaan, montakoLoydetty);
                lukuJotaEtsitaan = lista[i];
                montakoLoydetty = 1;
            }

            else
            {
                montakoLoydetty++;
            }
            i++;
        }

        alkiotJaMaarat.Add(lukuJotaEtsitaan, montakoLoydetty);

        IEnumerable<int> maarat = alkiotJaMaarat.OrderBy(x => x.Value).Select(x => x.Key);

        // Luodaan uusi taulukko, jonka pituus on alkuperaisessa taulukossa
        // olevien ERI alkioiden lukumaara. Kopioidaan sitten nama eri alkiot
        // taulukkoon. 
        int[] esiintymiskerrat = maarat.ToArray();
        return esiintymiskerrat;
    }

}