using System;
using System.Collections.Generic;
using System.Linq;

/// @author Antti-Jussi Lakanenm
/// @version 2025
/// 
/// <summary>
/// Etsitään pisin nouseva osajono.
/// </summary>
public class T1_PisinKasvava
{
    /// <summary>
    /// pääohjelma
    /// </summary>
    public static void Main()
    {
        int[] taulukko = { 3, 3, 4, 4, 4, 0, 3, 4, 6, 6, 2 };
        int jononPituus = PisinKasvava(taulukko);
        Console.WriteLine($"Pisimmän aidosti nousevan osajonon pituus on { jononPituus}");
        jononPituus = PisinKasvava2(taulukko);
        Console.WriteLine($"Pisimmän aidosti nousevan osajonon pituus on { jononPituus}");
        jononPituus = PisinKasvava3(taulukko);
        Console.WriteLine($"Pisimmän aidosti nousevan osajonon pituus on { jononPituus}");
    }

    
    
    /// <summary>
    /// Funktio etsii pisimmän aidosti kasvavan osajonon pituuden
    /// ja palauttaa sen.
    /// </summary>
    /// <param name="luvut">Luvut</param>
    /// <returns>Pisimmän aidosti kasvavan osajonon pituus.</returns>
    /// <example>
    /// <pre name="test">
    /// int[] luvut = { 3, 3, 4, 4, 4, 0, 3, 4, 6, 6, 2 };
    /// PisinKasvava(luvut) === 4;
    /// luvut = new int[] { 3, 3, 4, 4, 4, 0, 3, 4, 5, 6, 7 };
    /// PisinKasvava(luvut) === 6;
    /// luvut = new int[] { };
    /// PisinKasvava(luvut) === 0;
    /// PisinKasvava(new int[] { 1, 1, 1, 3, 5, 10 }) === 4;
    /// PisinKasvava(new int[] { 1 }) === 1;
    /// </pre>
    /// </example>
    public static int PisinKasvava(int[] luvut)
    {
        if (luvut.Length == 0) return 0;
        int pisin = 1;
        int pituusNyt = 1;
        for (int i = 1; i < luvut.Length; i++)
        {
            if (luvut[i] > luvut[i - 1])
            {
                pituusNyt++;
                if (pituusNyt > pisin) pisin = pituusNyt;
            }
            else pituusNyt = 1;
        }
        return pisin;
    }

    /// <summary>
    /// Funktio etsii pisimmän aidosti kasvavan osajonon pituuden
    /// ja palauttaa sen.
    /// </summary>
    /// <param name="luvut">Luvut</param>
    /// <returns>Pisimmän aidosti kasvavan osajonon pituus.</returns>
    /// <example>
    /// <pre name="test">
    /// int[] luvut = { 3, 3, 4, 4, 4, 0, 3, 4, 6, 6, 2 };
    /// PisinKasvava2(luvut) === 4;
    /// luvut = new int[] { 3, 3, 4, 4, 4, 0, 3, 4, 5, 6, 7 };
    /// PisinKasvava2(luvut) === 6;
    /// luvut = new int[] { };
    /// PisinKasvava2(luvut) === 0;
    /// PisinKasvava2(new int[] { 1, 1, 1, 3, 5, 10 }) === 4;
    /// PisinKasvava2(new int[] { 1 }) === 1;
    /// </pre>
    /// </example>
    public static int PisinKasvava2(int[] luvut)
    {
        if (luvut.Length == 0) return 0;
        List<List<int>> osajonot = new List<List<int>>();

        List<int> jonoNyt = new List<int>() { luvut[0]};
        for (int i = 1; i < luvut.Length; i++)
        {
            if (luvut[i] > luvut[i - 1]) jonoNyt.Add(luvut[i]);
            else
            {
                osajonot.Add(jonoNyt);
                jonoNyt = new List<int>() { luvut[i] };
            }
        }
        osajonot.Add(jonoNyt);

        int pisin = 0;
        for (int i = 0; i < osajonot.Count; i++)
        {
            if (osajonot[i].Count > pisin) pisin = osajonot[i].Count;
        }
        return pisin;
    }
    
    /// <summary>
    /// Funktio etsii pisimmän aidosti kasvavan osajonon pituuden
    /// ja palauttaa sen.
    /// </summary>
    /// <param name="luvut">Luvut</param>
    /// <returns>Pisimmän aidosti kasvavan osajonon pituus.</returns>
    /// <example>
    /// <pre name="test">
    /// int[] luvut = { 3, 3, 4, 4, 4, 0, 3, 4, 6, 6, 2 };
    /// PisinKasvava3(luvut) === 4;
    /// luvut = new int[] { 3, 3, 4, 4, 4, 0, 3, 4, 5, 6, 7 };
    /// PisinKasvava3(luvut) === 6;
    /// luvut = new int[] { };
    /// PisinKasvava3(luvut) === 0;
    /// PisinKasvava3(new int[] { 1, 1, 1, 3, 5, 10 }) === 4;
    /// PisinKasvava3(new int[] { 1 }) === 1;
    /// </pre>
    /// </example>
    public static int PisinKasvava3(int[] luvut)
    {
        if (luvut.Length == 0) return 0;

        var osajonot = new List<List<int>>();
        var jonoNyt = new List<int> { luvut[0] };
        osajonot.Add(jonoNyt);
        
        for (int i = 1; i < luvut.Length; i++)
        {
            // viimeisimmän osajonon viimeisin alkio
            // int viimeisin = osajonot.Last().Last();
            int viimeisin = jonoNyt[jonoNyt.Count - 1];

            if (luvut[i] > viimeisin)
            {
                // Päivitettään nykyistä osajonoa. Huomaa, että
                // tämän avulla tieto myös osajonot-listassa muuttuu
                // oikeaksi.
                jonoNyt.Add(luvut[i]);
            }
            else
            {
                jonoNyt = new List<int> { luvut[i] };
                osajonot.Add(jonoNyt);
            }
        }

        // Osajonojen suurin pituus on otettu tässä käyttäen LINQ:ta. 
        // Toki saman voisi tehdä "normaalisti" silmukallakin. 
        return osajonot.Select(osajono => osajono.Count).Max();
    }

    /// <summary>
    /// Funktio etsii pisimmän aidosti kasvavan osajonon pituuden
    /// ja palauttaa sen. Tässä ratkaisussa käytetään dynaamisen ohjelmoinnin lähestymistapaa. 
    /// </summary>
    /// <param name="luvut">Luvut</param>
    /// <returns>Pisimmän aidosti kasvavan osajonon pituus.</returns>
    /// <example>
    /// <pre name="test">
    /// int[] luvut = { 3, 3, 4, 4, 4, 0, 3, 4, 6, 6, 2 };
    /// PisinKasvava4(luvut) === 4;
    /// luvut = new int[] { 3, 3, 4, 4, 4, 0, 3, 4, 5, 6, 7 };
    /// PisinKasvava4(luvut) === 6;
    /// luvut = new int[] { };
    /// PisinKasvava4(luvut) === 0;
    /// PisinKasvava4(new int[] { 1, 1, 1, 3, 5, 10 }) === 4;
    /// PisinKasvava4(new int[] { 1 }) === 1;
    /// </pre>
    /// </example>
    public static int PisinKasvava4(int[] luvut)
    {
        if (luvut.Length == 0) return 0;
        int[] kasvu = new int[luvut.Length];
        Array.Fill(kasvu, 1);
        for (int i = 1; i < luvut.Length; i++)
            if (luvut[i] > luvut[i - 1]) kasvu[i] = kasvu[i - 1] + 1;
        // { 3, 3, 4, 4, 4, 0, 3, 4, 6, 6, 2 } // luvut
        // { 1, 1, 2, 1, 1, 1, 2, 3, 4, 1, 2 } // luvut
        return kasvu.Max();
    }
}
