using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

/// @author Antti-Jussi Lakanen
/// @version 2025
/// 
/// <summary>
/// Apuluokka 
/// </summary>
public static class Helper
{
    /// <summary>
    /// Laajennusmetodi (extension method): StringBuilder.IndexOf
    /// </summary>
    /// <param name="sb"></param>
    /// <param name="c"></param>
    /// <param name="startIndex"></param>
    /// <returns></returns>
    public static int IndexOf(this StringBuilder sb, char c, int startIndex = 0)
    {
        for (int i = startIndex; i < sb.Length; i++)
            if (sb[i] == c)
                return i;
        return -1;
    }
}

/// @author Antti-Jussi Lakanen
/// @version 2025
/// 
/// <summary>
/// Etsitään sanoja hauskasti.
/// </summary>
public class T7_Sanat
{
    /// <summary>
    /// Pääohjelmassa homma käyntiin
    /// </summary>
    /// <param name="args"></param>
    public static void Main(string[] args)
    {
        const string POLKU = "kotus_sanat.txt";
        string[] sanat = File.ReadAllLines(POLKU);

        #region Osa 1
        Console.Write("Anna merkkijono > ");
        string kayttajanSyote = Console.ReadLine();

        List<string> loytyneet = new List<string>();

        foreach (string jono in sanat)
        {
            if (SisaltaaKaikkiMerkit(jono, kayttajanSyote))
                loytyneet.Add(jono);
        }

        foreach (string loytynyt in loytyneet)
        {
            Console.WriteLine(loytynyt);
        }
        #endregion

        #region Osa 2
        string sana = "";

        while (true)
        {
            // Tekstin syötteen luku
            char c = Console.ReadKey().KeyChar;
            Console.Clear();
            if (c == 27) break; // escape
            if (c == '\b')
                sana = sana.Substring(0, Math.Max(sana.Length - 1, 0));
            else sana += c;
            Console.Write(sana);
            Console.WriteLine();
            // Löydettyjen tulostaminen 
            List<string> loydetyt = SisaltaaKaikkiMerkit(sanat, sana, Console.WindowHeight - 1);
            foreach (var loydetty in loydetyt)
            {
                TulostaSanaKorostuksella(loydetty, sana, ConsoleColor.Yellow);
                Console.WriteLine();
            }
            // Asetetaan kursori syöterivin loppuun
            Console.SetCursorPosition(sana.Length, 0);
        }
        #endregion
    }

    /// <summary>
    /// Onko anagrammi. Tämä ei ollut 
    /// tehtävänä, on vaan tässä muistutuksena.
    /// </summary>
    /// <param name="sana1">Ensimmäinen sana</param>
    /// <param name="sana2">Toinen sana</param>
    /// <returns></returns>
    /// <example>
    /// <pre name="test">
    ///  Anagrammi3("Antti", "ittnA") === true;
    ///  Anagrammi3("antti", "ittnA") === true;
    ///  Anagrammi3("an tti", "ittnA") === true;
    ///  Anagrammi3("an tti", "i ttnA") === true;
    ///  Anagrammi3("anttijussi", "I Just stina") === true;
    /// </pre>
    /// </example>
    public static bool Anagrammi3(string sana1, string sana2)
    {
        sana1 = sana1.ToLower().Replace(" ", ""); // Välilyönnit pois. Tämä voisi kyllä olla optiona (parametrina)
        sana2 = sana2.ToLower().Replace(" ", "");
        int[] frekvenssit1 = new int[char.MaxValue]; // char.MaxValue==65536
        int[] frekvenssit2 = new int[char.MaxValue];
        for (int i = 0; i < sana1.Length; i++)
            frekvenssit1[(int)sana1[i]]++;
        for (int i = 0; i < sana2.Length; i++)
            frekvenssit2[(int)sana2[i]]++;

        if (frekvenssit1.Length != frekvenssit2.Length) return false;

        for (int i = 0; i < frekvenssit1.Length; i++)
        {
            if (frekvenssit1[i] != frekvenssit2[i]) return false;
        }

        for (int i = 0; i < frekvenssit2.Length; i++)
        {
            if (frekvenssit1[i] != frekvenssit2[i]) return false;
        }

        return true;
    }

    /// <summary>
    /// Tulostaa sanan annetun värisellä korostuksella.
    /// Annetun "korostettavat"-jonon merkkien ensimmäiset 
    /// esiintymät "tulostettavaJono"-merkkijonossa tulostetaan
    /// korostusvärillä.
    /// </summary>
    /// <param name="tulostettavaJono">Mitä tulostetaan</param>
    /// <param name="korostettavat">Mitkä merkit korostetaan</param>
    /// <param name="korostusvari">Korostusväri</param>
    public static void TulostaSanaKorostuksella(string tulostettavaJono, string korostettavat, ConsoleColor korostusvari)
    {
        ConsoleColor valkoinen = ConsoleColor.Gray;
        ConsoleColor korostus = korostusvari;
        foreach (char tulostettavaMerkki in tulostettavaJono)
        {
            int tulostettavanPaikka = korostettavat.IndexOf(tulostettavaMerkki);
            if (tulostettavanPaikka >= 0)
            {
                Console.ForegroundColor = korostus;
                Console.Write(tulostettavaMerkki);
                Console.ForegroundColor = valkoinen;
                korostettavat = korostettavat.Remove(tulostettavanPaikka, 1);
            }
            else
            {
                Console.Write(tulostettavaMerkki);
            }
        }
    }

    /// <summary>
    /// Funktiossa tutkitaan voidaanko annettu merkkijono 
    /// upottaa toiseen merkkijonoon. Ensimmäisenä 
    /// parametrina sana johon upotetaan ja toisena upotettava sana.
    /// Tätä funktiota varten tehtiin stringBuilderille
    /// laajennusmetodi (extension method) IndexOf.
    /// </summary>
    /// <param name="sana">Sana, johon upotetaan</param>
    /// <param name="merkit">Upotettava sana (eli tämän sanan 
    /// kirjaimet pitää löytyä ekasta sanasta)</param>
    /// <returns>Onnistuko uptus</returns>
    /// <example>
    /// <pre name="test">
    /// SisaltaaKaikkiMerkit("antti", "antti") === true;
    /// SisaltaaKaikkiMerkit("antti", "at") === true;
    /// SisaltaaKaikkiMerkit("at", "antti") === false;
    /// SisaltaaKaikkiMerkit("anttijussi", "tia") === true;
    /// SisaltaaKaikkiMerkit("anttijussi", "") === true;
    /// SisaltaaKaikkiMerkit("anttijussi", "tiaa") === false;
    /// SisaltaaKaikkiMerkit("", "") === true;
    /// SisaltaaKaikkiMerkit("", "a") === false;
    /// </pre>
    /// </example>
    public static bool SisaltaaKaikkiMerkit(string sana, string merkit)
    {
        StringBuilder sanaNyt = new StringBuilder(sana);
        int i = 0;
        while (i < merkit.Length)
        {
            char merkkiNyt = merkit[i];
            int merkinPaikka = sanaNyt.IndexOf(merkkiNyt);
            if (merkinPaikka >= 0)
            {
                //sanaNyt[merkinPaikka] = (char)0;
                sanaNyt = sanaNyt.Remove(merkinPaikka, 1);
            }
            else return false;
            i++;
        }
        return true;
    }

    /// <summary>
    /// Antaa listan jonoista jotka löytyvät annetusta taulukosta
    /// </summary>
    /// <param name="jonot">Alkuperäinen aineisto</param>
    /// <param name="merkit">Mitkä merkit pitää löytyä</param>
    /// <param name="jonojaEnintaan">Montako merkkijonoa tulokseen enintään</param>
    /// <returns>Lista niistä jonoista jotka täyttävät annetun ehdon</returns>
    public static List<string> SisaltaaKaikkiMerkit(string[] jonot, string merkit, int jonojaEnintaan = int.MaxValue)
    {
        List<string> loydetytJonot = new List<string>();
        foreach (string jono in jonot)
        {
            if (SisaltaaKaikkiMerkit(jono, merkit)) loydetytJonot.Add(jono);
            if (loydetytJonot.Count == jonojaEnintaan) return loydetytJonot;
        }
        return loydetytJonot;
    }
}
